/** Globals **/
var LOOP_DELAY = 16;
var DEBUG = false;
var EMBED = false;
var PAUSE = false;
var MAX_BADDIES = 4;

// Game Globals
var score = 0;
var player;
var arena;

var gameState = "menu";
 
// Game Setup
function loadGame()
{
	//document.body.requestFullScreen();
	
	// init stuff
	arena = new Arena();
	
	player = new Player();
	
	score = 0;
	
	BaddieManager.init();
	
	HUD.init();
		
	attemptToStartGame();
		
}

function attemptToStartGame()
{
	console.log(ResourceManager.queue);
	if(!ResourceManager.isGameLoaded())
	{
		console.log("loading");
		
		setTimeout(attemptToStartGame, 10);
	}
	else
	{
		setupGame();
	}
}

function setupGame()
{
	// init Camera
	Camera.init();
	
	// set up handlers
	window.onresize = resizeHandler;
	
	window.onkeydown = KeyHandler.onKeyDown;
	window.onkeyup = KeyHandler.onKeyUp;
			
	gameState = "menu";
	
	// start game loop
	console.log("starting game loop");
	setInterval(gameLoop, LOOP_DELAY);
	
	//BaddieManager.spawnFreshBaddie();
	
	
	if(DEBUG)
		Camera.setScale(4);
}

function startGame()
{
	gameState = "on";
	BaddieManager.spawnFreshBaddie();		
}

function resetGame()
{
	var highScore = 0;
	
	if( localStorage["highScore"] != null)
	{
		var high = localStorage["highScore"];
		if(score > high)
			localStorage["highScore"] = score;
	}
	else
	{
		localStorage["highScore"] = score;
	}
	
	// init stuff
	arena = new Arena();
	
	player = new Player();
	
	score = 0;
	
	BaddieManager.init();
	
	gameState = "menu";		
}

function gameLoop()
{
	if (gameState == "menu")
	{
		redrawMenu();
	}
	else if(gameState == "on")
	{
		if(!PAUSE)
		{
			player.update();
	
			BaddieManager.updateAll();
		}
		
		redraw();
		
		if(PAUSE)
		{
			HUD.drawPause();
		}
		
	}
	else if (gameState == "over")
	{
		redrawOver();
	}
	
	
}

function resizeHandler(event)
{
	Camera.bestFitCamera();
}

function redraw()
{
	// draw background
	arena.drawGround();
	
	// draw bottom floor objects
	for(var i = 0; i < BaddieManager.corpses.length; i++)
	{
		BaddieManager.corpses[i].draw();
	}
	
	// put all the drawables together
	var drawables = getAllDrawables();

	drawables.sort(function(a, b) {
	  return a.position.y - b.position.y;
	});
	
	for(var i = 0; i < drawables.length; i++)
	{
		drawables[i].draw();
	}
	
	// draw effects
	for(var i = 0; i < drawables.length; i++)
	{
		if(drawables[i].drawEffects)
			drawables[i].drawEffects();
	}
	
	
	// draw player Light effect
	player.drawLight();
	
	HUD.draw();
}

function redrawMenu()
{
	// draw background
	arena.drawGround();
	
	player.draw();	
	
	// draw player Light effect
	player.drawLight();
		
	HUD.drawMenu();
}

function redrawOver()
{
	// draw background
	arena.drawGround();
	
	player.draw();	
	
	// draw player Light effect
	player.drawLight();
		
	HUD.drawGameOver();
}

function getAllDrawables()
{
	var drawables = [];
	
	for(var i = 0; i < BaddieManager.baddies.length; i++)
	{
		drawables.push(BaddieManager.baddies[i]);
	}
	
	drawables.push(player);
	
	return drawables;
}

function canBeAt(pos, obj)
{
	// check map objects
	
	// check baddie objects
	for(var i = 0; i < BaddieManager.corpses.length; i++)
	{
		if(BaddieManager.corpses[i] != obj)
			if(intersectsNew(pos, obj, BaddieManager.corpses[i]))
				return false;
	}
	
	// check baddies
	for(var i = 0; i < BaddieManager.baddies.length; i++)
	{
		if(BaddieManager.baddies[i] != obj)
			if(intersectsNew(pos, obj, BaddieManager.baddies[i]))
				return false;
	}
	
	// check player
	if(player != obj)
		if(intersectsNew(pos, obj, player))
			return false;
	
	return true;
};

function canBeAtCorpse(pos, obj)
{
	// check map objects
	
		
	// check baddies
	for(var i = 0; i < BaddieManager.baddies.length; i++)
	{
		if(BaddieManager.baddies[i] != obj)
			if(intersectsNew(pos, obj, BaddieManager.baddies[i]))
				return false;
	}
	
	// check player
	if(player != obj)
		if(intersectsNew(pos, obj, player))
			return false;
	
	return true;
};

function canBeAtZombie(pos, obj)
{
	// check map objects
	
		
	// check baddies
	for(var i = 0; i < BaddieManager.baddies.length; i++)
	{
		if(BaddieManager.baddies[i] != obj)
			if(intersectsNew(pos, obj, BaddieManager.baddies[i]))
				return false;
	}
	
	// check player
	if(player != obj)
		if(intersectsNew(pos, obj, player))
			return false;
	
	return true;
};

function intersectsNew(pos, obj, staticObj)
{
	
	if(obj.getRightBoundsFromPos(pos) <= staticObj.getLeftBounds())
	{
		return false;
	}
	
	if(obj.getLeftBoundsFromPos(pos) >= staticObj.getRightBounds())
	{
		return false;
	}
	
	if(obj.getBottomBoundsFromPos(pos) <= staticObj.getTopBounds())
	{
		return false;
	}
	
	if(obj.getTopBoundsFromPos(pos) >= staticObj.getBottomBounds())
	{
		return false;
	}
	
	return true;
}

function intersects(obj, staticObj)
{
	
	if(obj.getRightBounds() <= staticObj.getLeftBounds())
	{
		return false;
	}
	
	if(obj.getLeftBounds() >= staticObj.getRightBounds())
	{
		return false;
	}
	
	if(obj.getBottomBounds() <= staticObj.getTopBounds())
	{
		return false;
	}
	
	if(obj.getTopBounds() >= staticObj.getBottomBounds())
	{
		return false;
	}
	
	return true;
}

function intersectsZ(obj, staticObj, zTolerance)
{
	
	if(Math.abs(obj.position.y - staticObj.position.y) > zTolerance)
		return false;
	
	return true;
}