var CORPSE_ACCEL = 0.08;
var CORPSE_SPEED_FRICTION = 0.4;

function Corpse(type, pos, image)
{
	this.type = type;
	
	this.position = pos;
		
	this.vector = 90;
	
	this.velocity = 0;
	
	this.image = image;
	
	this.corpseBride = null;
	
	this.mergeRangeX = 12;
	this.mergeRangeY = 9;
	
	this.combining = false;
	this.combined = false;
	
	this.combineTimer = 700;
	this.combineStartTime = 0;
}

Corpse.prototype.update = function()
{
	if(this.combining)
	{
		if(this.isCombineReady() && this.combined == false)
			BaddieManager.combineCorpses(this, this.corpseBride);
	}	
	else if(this.corpseBride != null)
	{
		if (this.isInMergeRange())
		{
			this.beginCombination();		
		}
		else
		{
			// move towards bride if not in merge range
			this.vector = Util.angleTo(this.corpseBride.position, this.position);
			
			this.velocity += CORPSE_ACCEL;
		}
	}
	else	// is there another corpse to fall in love with?
	{
		for(var i = 0; i < BaddieManager.corpses.length; i++)
		{
			var corpse = BaddieManager.corpses[i];
			if(corpse != this && corpse.type == this.type && corpse.corpseBride == null)
			{
				this.corpseBride = corpse;
				corpse.corpseBride = this;
			}
		}
	}
	
	var newPos = {
		x: this.position.x - Math.cos(this.vector) * this.velocity,
		y: this.position.y - Math.sin(this.vector) * this.velocity
	};
	
	if(canBeAtCorpse({x: newPos.x, y: this.position.y}, this))
		this.position.x = newPos.x;	
	
	if(canBeAtCorpse({x: this.position.x, y: newPos.y}, this))
		this.position.y = newPos.y;	
	
	this.velocity -= this.velocity * CORPSE_SPEED_FRICTION;
};

Corpse.prototype.isCombineReady = function()
{
	if( Util.getTime() > this.combineStartTime + this.combineTimer)
		return true;
	
	return false;
};

Corpse.prototype.beginCombination = function()
{
	this.combining = true;
	this.combineStartTime = Util.getTime();
		
	// play combine SOUND
	BaddieManager.combineSound.play();
};

Corpse.prototype.isInMergeRange = function()
{
	var dX = Math.abs(this.position.x - this.corpseBride.position.x);
	var dY = Math.abs(this.position.y - this.corpseBride.position.y);
	
	if(dX <= this.mergeRangeX && dY <= this.mergeRangeY)
		return true;
	else
		return false;
};

Corpse.prototype.draw = function()
{
	Camera.drawImage(BaddieManager.dead, this.position.x - 11, this.position.y + 21, 35, 30);	
	
	// number
	Camera.drawImage(this.image, this.position.x, this.position.y+18, 17, 40);
	
	if(DEBUG)
	{
		var dx = this.position.x +  9 + Math.cos(this.vector ) * this.velocity * 8;
		var dy = this.position.y + 36 + Math.sin(this.vector ) * this.velocity * 8;
		
		Camera.drawLine("red", this.position.x + 9, this.position.y + 36, dx, dy);
		
		var vx = this.position.x +  9 - Math.cos(this.vector ) * 6;
		var vy = this.position.y + 36 - Math.sin(this.vector ) * 6;
		
		Camera.drawLine("green", this.position.x + 9, this.position.y + 36, vx, vy);
		
		Camera.drawLine("purple", this.getLeftBounds(), this.getTopBounds(), this.getRightBounds(), this.getTopBounds());
		Camera.drawLine("purple", this.getRightBounds(), this.getTopBounds(), this.getRightBounds(), this.getBottomBounds());
		Camera.drawLine("purple", this.getRightBounds(), this.getBottomBounds(), this.getLeftBounds(), this.getBottomBounds());
		Camera.drawLine("purple", this.getLeftBounds(), this.getBottomBounds(), this.getLeftBounds(), this.getTopBounds());
	
	}
};

var CORPSE_OFFSET_TOP = 31;
var CORPSE_OFFSET_BOTTOM = 39;
var CORPSE_OFFSET_LEFT = 3;
var CORPSE_OFFSET_RIGHT = 14;


Corpse.prototype.getTopBounds = function()
{
	return this.position.y + CORPSE_OFFSET_TOP;
};

Corpse.prototype.getBottomBounds = function()
{
	return this.position.y + CORPSE_OFFSET_BOTTOM;
};

Corpse.prototype.getLeftBounds = function()
{
	return this.position.x + CORPSE_OFFSET_LEFT;
};

Corpse.prototype.getRightBounds = function()
{
	return this.position.x + CORPSE_OFFSET_RIGHT;
};

Corpse.prototype.getTopBoundsFromPos = function(pos)
{
	return pos.y + CORPSE_OFFSET_TOP;
};

Corpse.prototype.getBottomBoundsFromPos = function(pos)
{
	return pos.y + CORPSE_OFFSET_BOTTOM;
};

Corpse.prototype.getLeftBoundsFromPos = function(pos)
{
	return pos.x + CORPSE_OFFSET_LEFT;
};

Corpse.prototype.getRightBoundsFromPos = function(pos)
{
	return pos.x + CORPSE_OFFSET_RIGHT;
};