var ZOMBIE_SPEED = 2;
var ZOMBIE_ACCEL = 0.2;
var ZOMBIE_SPEED_FRICTION = 0.4;

var ZOMBIE_TURN_SPEED = 10;


function Zombie(type, pos, image) 
{
	this.type = type;
	
	this.position = pos;
		
	this.vector = 90;
	
	this.velocity = 0;
	
	this.health = 1;
	this.dead = false;
	
	this.facing = "right";
	
	this.attacking = false;
	this.preppingAttack = false;

	this.attackRangeX = 18;
	this.attackRangeY = 4;

	this.prepTime = 500;
	this.attackTime = 200;
	this.attackResetTime = 1000;	
	
	this.spawnTime = 500;	
	
	this.prepStartTime = 0;
	this.attackStartTime = 0;
	this.attackEndTime = 0;
	
	this.spawnStartTime = Util.getTime();	

	// images	
	this.image = image;	
}

Zombie.prototype.update = function()
{
	if(this.dead)
		return;
		
	if(!this.fullySpawned())
		return;
	
	// if attacking
	if(this.attacking)
	{
		
		if(this.isAttackFinished())
			this.resetAttack();
	}
	
	// else if preparing to attack
	else if(this.preppingAttack)
	{	
		if(this.isAttackReady())
			this.attack();
			
		// keep prepping that attack bro
	}
	
	// else if in attack range
	else if (this.isInAttackRange() && this.canAttack())
	{
		// prepare to attack!
		//console.log("zombine prepping attack");
		this.prepAttack();
	}
	
	// else
	else if(!this.isInMinAttackRange())
	{
		// move towards the player if not in attack range
		this.vector = Util.angleTo(this.getAttackPosition(), this.position);
		
		this.velocity += ZOMBIE_ACCEL;
	}

	var newPos = {
		x: this.position.x - Math.cos(this.vector) * this.velocity,
		y: this.position.y - Math.sin(this.vector) * this.velocity
	};
	
	//aconsole.log(this.vector);
	//update facing
	this.facing = "right";
	
	// play prep SOUND
	
	if(player.position.x < this.position.x)
		this.facing = "left";
	
	if(canBeAtZombie({x: newPos.x, y: this.position.y}, this))
		this.position.x = newPos.x;	
	
	if(canBeAtZombie({x: this.position.x, y: newPos.y}, this))
		this.position.y = newPos.y;	
	
	this.velocity -= this.velocity * ZOMBIE_SPEED_FRICTION;
		
};

Zombie.prototype.fullySpawned = function()
{
	if( Util.getTime() > this.spawnStartTime + this.spawnTime)
		return true;
	
	return false;
};

Zombie.prototype.canAttack = function()
{
	if( Util.getTime() > this.attackEndTime + this.attackResetTime)
		return true;
	
	return false;
};

Zombie.prototype.prepAttack = function()
{
	this.preppingAttack = true;
	this.prepStartTime = Util.getTime();
	
	this.facing = "right";
	
	// play prep SOUND
	
	if(player.position.x < this.position.x)
		this.facing = "left";
};

Zombie.prototype.isAttackReady = function()
{
	if( Util.getTime() > this.prepStartTime + this.prepTime)
		return true;
	
	return false;
};

Zombie.prototype.attack = function()
{
	this.preppingAttack = false;
	
	this.attacking = true;
		
	this.attackStartTime = Util.getTime();
	
	var attackPos = {};
	
	if(this.facing == "left")
	{
		attackPos.x = this.position.x - 8 + 1;
		attackPos.y = this.position.y - 4 + 15;
	}	
	else
	{
		attackPos.x = this.position.x - 1 + 13;
		attackPos.y = this.position.y - 4 + 15;
	}	
	
	this.attackArea = new PosArea(attackPos, 12, 14);
	
	// check for hit
	if(intersectsZ(this, player, 4))
	{
		if(intersects(this.attackArea, player.getHitBox()))
		{
			// apply damage
			player.loseHealth(1);
			
			// play hit SOUND
			BaddieManager.hitSound.play();
		}
		else
		{
			// play miss sound
			BaddieManager.missSound.play();
		}
	}
	else
	{
		// play miss sound
		BaddieManager.missSound.play();
	}
};

Zombie.prototype.isAttackFinished = function()
{
	if( Util.getTime() > this.attackStartTime + this.attackTime)
		return true;
	
	return false;
};

Zombie.prototype.resetAttack = function()
{
	this.attacking = false;
	
	this.attackEndTime = Util.getTime();
	
	this.attackArea = null;
};

Zombie.prototype.getAttackPosition = function()
{
	var outOFView = false;
	// get in view first please
	if(this.position.x > 480-8)
		outOFView = true;
	else if(this.position.x < -8)
		outOFView = true;
		
	if(this.position.y > 270-20)
		outOFView = true;
	else if(this.position.y < -20)
		outOFView = true;
	
	if(outOFView)
		return {x: 232, y: 115};
	
	var x = player.position.x;
	
	if(this.position.x < player.position.x)
	{
		x -= 12;
	}
	else
	{
		x += 12;
	}
	var y = player.position.y;
	
	return {x: x, y: y};
};

Zombie.prototype.isInAttackRange = function()
{
	var dX = Math.abs(this.position.x - player.position.x);
	var dY = Math.abs(this.position.y - player.position.y);
	
	if(dX <= this.attackRangeX && dY <= this.attackRangeY)
		return true;
	else
		return false;
};

Zombie.prototype.isInMinAttackRange = function()
{
	var dX = Math.abs(this.position.x - player.position.x);
	var dY = Math.abs(this.position.y - player.position.y);
	
	if(dX <= this.attackRangeX - 6 && dY <= this.attackRangeY)
		return true;
	else
		return false;
};

Zombie.prototype.loseHealth = function(dmg)
{	
	this.health -= dmg;
	
	if(this.health == 0)
		this.die();
};

Zombie.prototype.die = function()
{
	this.dead = true;
	BaddieManager.dealWithDeath(this);
};

Zombie.prototype.getHitBox = function()
{
	var pos = {};
	pos.x = this.position.x + 3;
	pos.y = this.position.y + 2;
	
	var w = 11;
	var h = 34;
	
	return new PosArea(pos, w, h);
};

Zombie.prototype.draw = function()
{
	
	//guy shadow
	Camera.drawImage(BaddieManager.shadow, this.position.x - 3, this.position.y + 33, 21, 12);
		
	
		
	// guy	
	if(this.attacking)
	{
		if(this.facing == "left")
			Camera.drawImage(BaddieManager.guyAttack_left, this.position.x-8, this.position.y-4, 26, 44);
		else if(this.facing == "right")
			Camera.drawImage(BaddieManager.guyAttack_right, this.position.x-1, this.position.y-4, 26, 44);
	}
	else if(this.preppingAttack)
	{
		if(this.facing == "left")
			Camera.drawImage(BaddieManager.guyPrep_left, this.position.x, this.position.y, 19, 40);
		else if(this.facing == "right")
			Camera.drawImage(BaddieManager.guyPrep_right, this.position.x, this.position.y, 19, 40);
	}
	else
	{
		if(this.facing == "left")
			Camera.drawImage(BaddieManager.guyLeft, this.position.x, this.position.y, 17, 40);
		else if(this.facing == "right")
			Camera.drawImage(BaddieManager.guyRight, this.position.x, this.position.y, 17, 40);
	}
	// number
	Camera.drawImage(this.image, this.position.x, this.position.y, 17, 40);
	
	
	if(DEBUG)
	{
		var dx = this.position.x +  9 + Math.cos(this.vector ) * this.velocity * 8;
		var dy = this.position.y + 16 + Math.sin(this.vector ) * this.velocity * 8;
		
		Camera.drawLine("red", this.position.x + 9, this.position.y + 16, dx, dy);
		
		var vx = this.position.x +  9 - Math.cos(this.vector ) * 6;
		var vy = this.position.y + 16 - Math.sin(this.vector ) * 6;
		
		Camera.drawLine("green", this.position.x + 9, this.position.y + 16, vx, vy);
		
		Camera.drawLine("purple", this.getLeftBounds(), this.getTopBounds(), this.getRightBounds(), this.getTopBounds());
		Camera.drawLine("purple", this.getRightBounds(), this.getTopBounds(), this.getRightBounds(), this.getBottomBounds());
		Camera.drawLine("purple", this.getRightBounds(), this.getBottomBounds(), this.getLeftBounds(), this.getBottomBounds());
		Camera.drawLine("purple", this.getLeftBounds(), this.getBottomBounds(), this.getLeftBounds(), this.getTopBounds());
	
		var box = this.getHitBox();
		Camera.drawLine("blue", box.getLeftBounds(), box.getTopBounds(), box.getRightBounds(), box.getTopBounds());
		Camera.drawLine("blue", box.getRightBounds(), box.getTopBounds(), box.getRightBounds(), box.getBottomBounds());
		Camera.drawLine("blue", box.getRightBounds(), box.getBottomBounds(), box.getLeftBounds(), box.getBottomBounds());
		Camera.drawLine("blue", box.getLeftBounds(), box.getBottomBounds(), box.getLeftBounds(), box.getTopBounds());
		
	
	}
};

Zombie.prototype.drawEffects = function()
{
	//attacl swoosh
	if(this.attacking)
	{
		if(this.facing == "left")
			Camera.drawImage(BaddieManager.guyAttackSwoosh_left, this.position.x-8, this.position.y-4, 26, 44);
		else if(this.facing == "right")
			Camera.drawImage(BaddieManager.guyAttackSwoosh_right, this.position.x-1, this.position.y-4, 26, 44);
	}
	
	if(DEBUG)
	{
		if(this.attackArea)
		{
			Camera.drawLine("red", this.attackArea.getLeftBounds(), this.attackArea.getTopBounds(), this.attackArea.getRightBounds(), this.attackArea.getTopBounds());
			Camera.drawLine("red", this.attackArea.getRightBounds(), this.attackArea.getTopBounds(), this.attackArea.getRightBounds(), this.attackArea.getBottomBounds());
			Camera.drawLine("red", this.attackArea.getRightBounds(), this.attackArea.getBottomBounds(), this.attackArea.getLeftBounds(), this.attackArea.getBottomBounds());
			Camera.drawLine("red", this.attackArea.getLeftBounds(), this.attackArea.getBottomBounds(), this.attackArea.getLeftBounds(), this.attackArea.getTopBounds());
		}
	
	}
};

//var DEFAULT_GUY_OFFSET_TOP	 	= 20;

var DEFAULT_GUY_OFFSET_TOP	 	= 36;
var DEFAULT_GUY_OFFSET_BOTTOM	= 42;
var DEFAULT_GUY_OFFSET_LEFT		= 3;
var DEFAULT_GUY_OFFSET_RIGHT	= 14;

Zombie.prototype.getTopBounds = function()
{
	return this.position.y + DEFAULT_GUY_OFFSET_TOP;
};

Zombie.prototype.getBottomBounds = function()
{
	return this.position.y + DEFAULT_GUY_OFFSET_BOTTOM;
};

Zombie.prototype.getLeftBounds = function()
{
	return this.position.x + DEFAULT_GUY_OFFSET_LEFT;
};

Zombie.prototype.getRightBounds = function()
{
	return this.position.x + DEFAULT_GUY_OFFSET_RIGHT;
};

Zombie.prototype.getTopBoundsFromPos = function(pos)
{
	return pos.y + DEFAULT_GUY_OFFSET_TOP;
};

Zombie.prototype.getBottomBoundsFromPos = function(pos)
{
	return pos.y + DEFAULT_GUY_OFFSET_BOTTOM;
};

Zombie.prototype.getLeftBoundsFromPos = function(pos)
{
	return pos.x + DEFAULT_GUY_OFFSET_LEFT;
};

Zombie.prototype.getRightBoundsFromPos = function(pos)
{
	return pos.x + DEFAULT_GUY_OFFSET_RIGHT;
};

